/*-
 *  Copyright (c) 2004-2005  Peter Pentchev
 *  All rights reserved.
 * 
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */

/* $Ringlet: c/misc/disperse/disperse.c,v 1.2 2005/01/03 09:43:25 roam Exp $ */

#include <err.h>
#include <stdio.h>
#include <string.h>

#ifndef __dead2
#ifdef __GNUC__
#define	__dead2	__attribute__((noreturn))
#else
#define	__dead2
#endif /* __GNUC__ */
#endif /* __dead2 */

static void	 usage(void);
static void	 disperse(FILE *, FILE *);

/*
 * Show usage information
 */
static void __dead2
usage(void)
{
	errx(1, "Usage: disperse infile outfile");
}

/*
 * Disperse a single file
 */
void
disperse(FILE *in, FILE *out)
{
	long skipcnt;
	int ch;

	skipcnt = 0;
	while ((ch = fgetc(in)) != EOF) {
		if (ch == '\0') {
			skipcnt++;
		} else {
			if (skipcnt > 0) {
				if (fseek(out, skipcnt, SEEK_CUR) == -1)
					err(1, "extending output file");
				skipcnt = 0;
			}
			if (fputc(ch, out) == EOF)
				err(1, "writing output file");
		}
#ifdef ASSERTIONS
		if (ftell(in) - ftell(out) - skipcnt != 0) {
			fprintf(stderr, "Bad things at pos %ld: %ld\n",
			    (long)ftell(in),
			    (long)(ftell(in) - ftell(out) - skipcnt));
			abort();
		}
#endif /* ASSERTIONS */
	}
	if (ferror(in))
		err(1, "reading input file");
	if (skipcnt > 0) {
		if (fseek(out, skipcnt - 1, SEEK_CUR) == -1)
			err(1, "extending output file");
		skipcnt = 0;
		if (fwrite(&skipcnt, 1, 1, out) != 1)
			err(1, "writing output file");
	}
}

/*
 * Examine the command-line arguments and disperse the files specified
 */
int
main(int argc, char *argv[])
{
	FILE *in, *out;
	int clin, clout;

	/* Examine the command-line arguments */
	if (argc == 1) {
		clin = clout = 0;
	} else if (argc == 3) {
		clin = strcmp(argv[1], "-");
		clout = strcmp(argv[2], "-");
	} else {
		usage();
	}
	
	/* Open the files as needed */
	if (clin)
		in = fopen(argv[1], "rb");
	else
		in = stdin;
	if (in == NULL)
		err(1, "opening input file");
	if (clout)
		out = fopen(argv[2], "wb");
	else
		out = stdout;
	if (out == NULL)
		err(1, "opening output file");
	
	/* Rock'n'roll */
	disperse(in, out);

	if (clin)
		fclose(in);
	if (clout)
		fclose(out);
	return (0);
}
